@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2023 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -40,8 +40,7 @@ class GpxDocument(QObject):
     self.trkmodel = self.gpxparser.trkmodel
     self.tmpdir = mkdtemp(prefix='gpxviewer_tmpdir.')
     self.projectType = TYPE_GPXV
-    self.doc = {}
-    self.doc['GPXFile'] = []
+    self.doc = {'GPXFile': []}
 
   def close(self):
     shutil.rmtree(self.tmpdir)
@@ -74,6 +73,8 @@ class GpxDocument(QObject):
     self.doc['CaptionPositionYs'] = self.wptmodel.getPointStyles(gpx.CAPTION_POSY)
     self.doc['CaptionRotation'] = self.wptmodel.getPointStyles(gpx.CAPTION_ROTATION)
     self.doc['CaptionSizes'] = self.wptmodel.getPointStyles(gpx.CAPTION_SIZE)
+    self.doc['CaptionBold'] = self.wptmodel.getPointStyles(gpx.CAPTION_BOLD)
+    self.doc['CaptionItalic'] = self.wptmodel.getPointStyles(gpx.CAPTION_ITALIC)
 
     self.doc['ChangedNames'] = list(self.wptmodel.changedNames.keys())
     self.doc['PointNames'] = list(self.wptmodel.changedNames.values())
@@ -238,6 +239,12 @@ class GpxDocument(QObject):
       if 'CaptionPoints' in self.doc and 'CaptionSizes' in self.doc:
         for i, m in zip(self.doc['CaptionPoints'], self.doc['CaptionSizes']):
           self.wptmodel.setPointStyle([i], gpx.CAPTION_SIZE, m)
+      if 'CaptionPoints' in self.doc and 'CaptionBold' in self.doc:
+        for i, m in zip(self.doc['CaptionPoints'], self.doc['CaptionBold']):
+          self.wptmodel.setPointStyle([i], gpx.CAPTION_BOLD, m)
+      if 'CaptionPoints' in self.doc and 'CaptionItalic' in self.doc:
+        for i, m in zip(self.doc['CaptionPoints'], self.doc['CaptionItalic']):
+          self.wptmodel.setPointStyle([i], gpx.CAPTION_ITALIC, m)
 
       if 'SplitLines' in self.doc and 'SplitLineColors' in self.doc:
         for i, m in zip(self.doc['SplitLines'], self.doc['SplitLineColors']):
@@ -251,7 +258,7 @@ class GpxDocument(QObject):
 
       if 'ChangedNames' in self.doc and 'PointNames' in self.doc:
         for i, n in zip(self.doc['ChangedNames'], self.doc['PointNames']):
-          self.wptmodel.setData(self.wptmodel.index(i, gpx.NAME), n, Qt.EditRole)
+          self.wptmodel.setData(self.wptmodel.index(i, gpx.NAME), n, Qt.ItemDataRole.EditRole)
       if 'ChangedAltitudes' in self.doc and 'PointAltitudes' in self.doc:
         self.wptmodel.setAltitudes(self.doc['ChangedAltitudes'], self.doc['PointAltitudes'])
 
