@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2023 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -18,7 +18,7 @@ import sys
 import re
 import webbrowser
 from PyQt5.QtCore import Qt, QCoreApplication, QEvent, QFileInfo, QFileSelector, QUrl, QT_VERSION, QT_VERSION_STR, PYQT_VERSION_STR, pyqtSlot
-from PyQt5.QtGui import QCursor, QIcon, QKeyEvent, QKeySequence
+from PyQt5.QtGui import QCursor, QIcon, QKeySequence
 from PyQt5.QtNetwork import QNetworkAccessManager, QNetworkReply, QNetworkRequest
 from PyQt5.QtWidgets import (QAction, QApplication, QDialog, QDialogButtonBox, QFileDialog, QHBoxLayout, QLabel, QLineEdit,
                              QMainWindow, QMenu, QMessageBox, QSizePolicy, QSpacerItem, QSpinBox, QVBoxLayout, QWidget)
@@ -44,7 +44,7 @@ class GpxMainWindow(QMainWindow):
 
     wdg = QWidget()
     wdg.setLayout(QHBoxLayout())
-    wdg.layout().addItem(QSpacerItem(40, 20, QSizePolicy.Expanding))
+    wdg.layout().addItem(QSpacerItem(40, 20, QSizePolicy.Policy.Expanding))
     wdg.layout().addWidget(QLabel(self.tr('Filter by name:')))
     self.filterLineEdit = QLineEdit()
     self.filterLineEdit.setMinimumWidth(200)
@@ -56,12 +56,12 @@ class GpxMainWindow(QMainWindow):
     self.filterModel = gpx.GpxSortFilterModel(self)
     self.filterModel.setSourceModel(TheDocument.wptmodel)
     self.filterModel.setFilterKeyColumn(gpx.NAME)
-    self.filterLineEdit.textChanged.connect(self.filterModel.setFilterRegExp)
+    self.filterLineEdit.textChanged.connect(self.filterModel.setFilterRegularExpression)
     self.updateIncludeFilter()
 
     self.ui.wptView.setModel(self.filterModel)
     self.ui.wptView.setSortingEnabled(True)
-    self.ui.wptView.sortByColumn(gpx.TIME, Qt.AscendingOrder)
+    self.ui.wptView.sortByColumn(gpx.TIME, Qt.SortOrder.AscendingOrder)
     self.ui.trkView.setModel(TheDocument.trkmodel)
     self.ui.wptView.installEventFilter(self)
     self.ui.trkView.installEventFilter(self)
@@ -97,6 +97,7 @@ class GpxMainWindow(QMainWindow):
 
     self.plotWindow = plt.PlotWindow()
     self.plotWindow.profileChanged.connect(self.setProjectChanged)
+    self.plotWindow.gotoMainWindow[int].connect(self.onGotoMainWindow)
     self.statWindow = stat.StatWindow()
 
   def updateTheme(self):
@@ -122,46 +123,46 @@ class GpxMainWindow(QMainWindow):
     self.ui.actionAboutGPXViewer.setIcon(QIcon(':/icons/gpxviewer.svg'))
 
   def initActions(self):
-    self.ui.actionNew.setShortcut(QKeySequence.New)
-    self.ui.actionOpen.setShortcut(QKeySequence.Open)
-    self.ui.actionSave.setShortcut(QKeySequence.Save)
-    self.ui.actionSaveAs.setShortcut(QKeySequence.SaveAs)
-    self.ui.actionQuit.setShortcut(QKeySequence.Quit)
-    self.ui.actionCopy.setShortcut(QKeySequence.Copy)
-    self.ui.actionSettings.setShortcut(QKeySequence.Preferences)
-    self.ui.actionGpxViewerHelp.setShortcut(QKeySequence.HelpContents)
+    self.ui.actionNew.setShortcut(QKeySequence.StandardKey.New)
+    self.ui.actionOpen.setShortcut(QKeySequence.StandardKey.Open)
+    self.ui.actionSave.setShortcut(QKeySequence.StandardKey.Save)
+    self.ui.actionSaveAs.setShortcut(QKeySequence.StandardKey.SaveAs)
+    self.ui.actionQuit.setShortcut(QKeySequence.StandardKey.Quit)
+    self.ui.actionCopy.setShortcut(QKeySequence.StandardKey.Copy)
+    self.ui.actionSettings.setShortcut(QKeySequence.StandardKey.Preferences)
+    self.ui.actionGpxViewerHelp.setShortcut(QKeySequence.StandardKey.HelpContents)
 
     self.actSkipPoints = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip points'), self)
     self.actSkipPoints.setStatusTip(self.tr('Skip these waypoints when plotting profiles or calculating statistics'))
-    self.actSkipPoints.setShortcut(Qt.Key_Delete)
+    self.actSkipPoints.setShortcut(QKeySequence(Qt.Key.Key_Delete))
     self.actSkipPoints.setCheckable(True)
     self.actSkipPoints.triggered.connect(self.skipPoints)
     self.addAction(self.actSkipPoints)
 
     self.actMarker = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-marker.svg')), self.tr('Points with markers'), self)
     self.actMarker.setStatusTip(self.tr('Add markers to these waypoints when plotting profiles'))
-    self.actMarker.setShortcut(Qt.Key_M)
+    self.actMarker.setShortcut(QKeySequence(Qt.Key.Key_M))
     self.actMarker.setCheckable(True)
     self.actMarker.triggered.connect(self.markerPoints)
     self.addAction(self.actMarker)
 
     self.actCaption = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-caption.svg')), self.tr('Points with captions'), self)
     self.actCaption.setStatusTip(self.tr('Add captions to these waypoints when plotting profiles'))
-    self.actCaption.setShortcut(Qt.Key_C)
+    self.actCaption.setShortcut(QKeySequence(Qt.Key.Key_C))
     self.actCaption.setCheckable(True)
     self.actCaption.triggered.connect(self.captionPoints)
     self.addAction(self.actCaption)
 
     self.actSplit = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-splitline.svg')), self.tr('Points with splitting lines'), self)
     self.actSplit.setStatusTip(self.tr('Add splitting lines to these waypoints when plotting profiles'))
-    self.actSplit.setShortcut(Qt.Key_S)
+    self.actSplit.setShortcut(QKeySequence(Qt.Key.Key_S))
     self.actSplit.setCheckable(True)
     self.actSplit.triggered.connect(self.splitLines)
     self.addAction(self.actSplit)
 
     self.actNeglect = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-neglect-distance.svg')), self.tr('Neglect previous distance'), self)
     self.actNeglect.setStatusTip(self.tr('Neglect distance before these waypoints when plotting profiles'))
-    self.actNeglect.setShortcut(Qt.Key_N)
+    self.actNeglect.setShortcut(QKeySequence(Qt.Key.Key_N))
     self.actNeglect.setCheckable(True)
     self.actNeglect.triggered.connect(self.neglectDistance)
     self.addAction(self.actNeglect)
@@ -173,7 +174,7 @@ class GpxMainWindow(QMainWindow):
 
     self.actRename = QAction(QIcon(self.themeSelector.select(':/icons/edit-rename.svg')), self.tr('Rename...'), self)
     self.actRename.setStatusTip(self.tr('Rename these waypoints'))
-    self.actRename.setShortcut(Qt.Key_F2)
+    self.actRename.setShortcut(QKeySequence(Qt.Key.Key_F2))
     self.actRename.triggered.connect(self.renamePoints)
     self.addAction(self.actRename)
 
@@ -184,7 +185,7 @@ class GpxMainWindow(QMainWindow):
 
     self.actStyle = QAction(QIcon(self.themeSelector.select(':/icons/configure.svg')), self.tr('Point style'), self)
     self.actStyle.setStatusTip(self.tr('Change the style of these waypoints when plotting profiles'))
-    self.actStyle.setShortcut(Qt.ALT + Qt.Key_Return)
+    self.actStyle.setShortcut(QKeySequence(Qt.Modifier.ALT | Qt.Key.Key_Return))
     self.actStyle.triggered.connect(self.pointStyle)
     self.addAction(self.actStyle)
 
@@ -200,7 +201,7 @@ class GpxMainWindow(QMainWindow):
 
     self.actSkipTracks = QAction(QIcon(self.themeSelector.select(':/icons/waypoint-skip.svg')), self.tr('Skip tracks'), self)
     self.actSkipTracks.setStatusTip(self.tr('Skip these tracks when plotting profiles or calculating statistics'))
-    self.actSkipTracks.setShortcut(Qt.Key_Delete)
+    self.actSkipTracks.setShortcut(QKeySequence(Qt.Key.Key_Delete))
     self.actSkipTracks.setCheckable(True)
     self.actSkipTracks.triggered.connect(self.skipTracks)
     self.addAction(self.actSkipTracks)
@@ -221,21 +222,21 @@ class GpxMainWindow(QMainWindow):
 
   def eventFilter(self, obj, event):
     if obj == self.ui.wptView:
-      if event.type() == QEvent.ContextMenu or (event.type() == QEvent.KeyPress and event.key() == Qt.Key_Menu and event.modifiers() == Qt.NoModifier):
+      if event.type() == QEvent.Type.ContextMenu or (event.type() == QEvent.Type.KeyPress and event.key() == Qt.Key.Key_Menu and event.modifiers() == Qt.KeyboardModifier.NoModifier):
         self.wptContextMenuEvent(event)
         return True
 
-      elif event.type() == QEvent.KeyPress:
-        self.keyPressEvent(QKeyEvent(event))
+      elif event.type() == QEvent.Type.KeyPress:
+        self.keyPressEvent(event)
         # Do not return True
 
     elif obj == self.ui.trkView:
-      if event.type() == QEvent.ContextMenu or (event.type() == QEvent.KeyPress and event.key() == Qt.Key_Menu and event.modifiers() == Qt.NoModifier):
+      if event.type() == QEvent.Type.ContextMenu or (event.type() == QEvent.Type.KeyPress and event.key() == Qt.Key.Key_Menu and event.modifiers() == Qt.KeyboardModifier.NoModifier):
         self.trkContextMenuEvent(event)
         return True
 
-      elif event.type() == QEvent.KeyPress:
-        self.keyPressEvent(QKeyEvent(event))
+      elif event.type() == QEvent.Type.KeyPress:
+        self.keyPressEvent(event)
         # Do not return True
 
     return super(GpxMainWindow, self).eventFilter(obj, event)
@@ -279,30 +280,20 @@ class GpxMainWindow(QMainWindow):
       actShowGoogleMap = QAction(QIcon(':/icons/googlemaps.png'), self.tr('Google Maps'), self)
       actShowGoogleMap.setStatusTip(self.tr('Show this waypoint on the website maps.google.com'))
       actShowGoogleMap.triggered.connect(self.showGoogleMap)
+
       actShowYandexMap = QAction(QIcon(':/icons/yandexmaps.png'), self.tr('Yandex Maps'), self)
       actShowYandexMap.setStatusTip(self.tr('Show this waypoint on the website maps.yandex.ru'))
       actShowYandexMap.triggered.connect(self.showYandexMap)
-      actShowZoomEarthMap = QAction(QIcon(':/icons/zoomearth.png'), self.tr('Zoom Earth'), self)
-      actShowZoomEarthMap.setStatusTip(self.tr('Show this waypoint on the website zoom.earth'))
-      actShowZoomEarthMap.triggered.connect(self.showZoomEarthMap)
-      actShowOpenCycleMap = QAction(QIcon(':/icons/openstreetmap.png'), self.tr('OpenCycleMap'), self)
-      actShowOpenCycleMap.setStatusTip(self.tr('Show this waypoint on the website openstreetmap.org'))
-      actShowOpenCycleMap.triggered.connect(self.showOpenCycleMap)
-      actShowOpenTopoMap = QAction(QIcon(':/icons/opentopomap.png'), self.tr('OpenTopoMap'), self)
-      actShowOpenTopoMap.setStatusTip(self.tr('Show this waypoint on the website opentopomap.org'))
-      actShowOpenTopoMap.triggered.connect(self.showOpenTopoMap)
-      actShowTopoMap = QAction(QIcon(':/icons/loadmap.png'), self.tr('Loadmap.net'), self)
-      actShowTopoMap.setStatusTip(self.tr('Show this waypoint on the website loadmap.net'))
-      actShowTopoMap.triggered.connect(self.showTopoMap)
+
+      actShowNakarteMap = QAction(QIcon(':/icons/nakarte.png'), self.tr('Nakarte.me'), self)
+      actShowNakarteMap.setStatusTip(self.tr('Show this waypoint on the website nakarte.me'))
+      actShowNakarteMap.triggered.connect(self.showNakarteMap)
 
       showMapMenu = QMenu(self.tr('Show on map'), self)
       showMapMenu.setIcon(QIcon(self.themeSelector.select(':/icons/internet-services.svg')))
       showMapMenu.addAction(actShowGoogleMap)
       showMapMenu.addAction(actShowYandexMap)
-      showMapMenu.addAction(actShowZoomEarthMap)
-      showMapMenu.addAction(actShowOpenCycleMap)
-      showMapMenu.addAction(actShowOpenTopoMap)
-      showMapMenu.addAction(actShowTopoMap)
+      showMapMenu.addAction(actShowNakarteMap)
       menu.addMenu(showMapMenu)
 
       menu.popup(QCursor.pos())
@@ -321,11 +312,11 @@ class GpxMainWindow(QMainWindow):
       event.accept()
 
   def keyPressEvent(self, event):
-    if event.matches(QKeySequence.Find):
+    if event.matches(QKeySequence.StandardKey.Find):
       self.filterLineEdit.setFocus()
       self.filterLineEdit.selectAll()
 
-    elif event.key() == Qt.Key_Escape:
+    elif event.key() == Qt.Key.Key_Escape:
       if self.ui.tabWidget.currentWidget() == self.ui.wptTab:
         if self.ui.wptView.hasFocus():
           self.ui.wptView.clearSelection()
@@ -337,7 +328,7 @@ class GpxMainWindow(QMainWindow):
         else:
           self.ui.trkView.setFocus()
 
-    elif event.key() == Qt.Key_Tab and event.modifiers() == Qt.ControlModifier:
+    elif event.key() == Qt.Key.Key_Tab and event.modifiers() == Qt.KeyboardModifier.ControlModifier:
       self.ui.tabWidget.setCurrentIndex(1 - self.ui.tabWidget.currentIndex())
 
     super(GpxMainWindow, self).keyPressEvent(event)
@@ -368,9 +359,9 @@ class GpxMainWindow(QMainWindow):
   def onAboutGPXViewer(self):
     aboutText = '<h3>GPX Viewer</h3><b>' + self.tr('Version') + ' ' + QCoreApplication.applicationVersion() + '</b><br><br>' + \
                 self.tr('Using') + ' Python ' + str(sys.version_info.major) + '.' + str(sys.version_info.minor) + '.' + str(sys.version_info.micro) + ', ' + \
-                'PyQt5 ' + PYQT_VERSION_STR + ', ' + \
+                'PyQt' + PYQT_VERSION_STR[0] + ' ' + PYQT_VERSION_STR + ', ' + \
                 'Qt ' + QT_VERSION_STR + '<br><br>' + \
-                'Copyright 2016-2020 Sergey Salnikov <a href=mailto:salsergey@gmail.com>&lt;salsergey@gmail.com&gt;</a><br><br>' + \
+                'Copyright 2016-2023 Sergey Salnikov <a href=mailto:salsergey@gmail.com>&lt;salsergey@gmail.com&gt;</a><br><br>' + \
                 self.tr('License:') + ' <a href=http://www.gnu.org/licenses/gpl.html>GNU General Public License, version 3</a>'
     QMessageBox.about(self, self.tr('About GPX Viewer'), aboutText)
 
@@ -397,6 +388,7 @@ class GpxMainWindow(QMainWindow):
                            Several interactions are possible in profile window:
                            <ul>
                            <li>Left/right mouse buttons select markers or captions</li>
+                           <li>Hold Ctrl button to select multiple points</li>
                            <li>Selected captions are moved by up/down/left/right buttons (with or without Ctrl)</li>
                            <li>Selected markers are changed by left/right buttons (with or without Ctrl)</li>
                            <li>Mouse wheel zooms the image horizontally</li>
@@ -405,10 +397,10 @@ class GpxMainWindow(QMainWindow):
                            </ul>''')
     msg = QMessageBox(self)
     msg.setWindowTitle(self.tr('GPX Viewer Help'))
-    msg.setTextFormat(Qt.RichText)
+    msg.setTextFormat(Qt.TextFormat.RichText)
     msg.setText(aboutText)
-    msg.setIcon(QMessageBox.Information)
-    msg.exec_()
+    msg.setIcon(QMessageBox.Icon.Information)
+    msg.exec()
 
   @pyqtSlot()
   @pyqtSlot(bool)
@@ -479,18 +471,18 @@ class GpxMainWindow(QMainWindow):
     numberBox.setValue(1)
     hLayout.addWidget(numberBox)
 
-    buttons = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+    buttons = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
     buttons.accepted.connect(dlg.accept)
     buttons.rejected.connect(dlg.reject)
     vLayout.addWidget(buttons)
 
-    if dlg.exec_() == QDialog.Accepted and len(nameEdit.text()) > 0:
+    if dlg.exec() == QDialog.DialogCode.Accepted and len(nameEdit.text()) > 0:
       for n, ind in enumerate(indexes, numberBox.value()):
         name = nameEdit.text()
         if re.match('.*#+', name):
           digits = len(re.findall('(#+)', name)[0])
           name = re.sub('#+', str(n).zfill(digits), name, 1)
-        TheDocument.wptmodel.setData(TheDocument.wptmodel.index(ind, gpx.NAME), name, Qt.EditRole)
+        TheDocument.wptmodel.setData(TheDocument.wptmodel.index(ind, gpx.NAME), name, Qt.ItemDataRole.EditRole)
 
   @pyqtSlot()
   def resetPointNames(self):
@@ -517,13 +509,15 @@ class GpxMainWindow(QMainWindow):
 
   @pyqtSlot(QNetworkReply)
   def replyReady(self, reply):
-    if reply.error() != QNetworkReply.NoError and self.showNetworkError:
+    reply.deleteLater()
+    if reply.error() != QNetworkReply.NetworkError.NoError and self.showNetworkError:
       self.showNetworkError = False
       QMessageBox.warning(self, self.tr('Network error'),
                           self.tr('Error requesting SRTM data from http://geonames.org.') + '\n\n' + reply.errorString())
 
     else:
       request = reply.request().url().toString()
+      line = ''
       try:
         indexes = self.requestedIndexes.pop(request)
         alts = []
@@ -573,6 +567,19 @@ class GpxMainWindow(QMainWindow):
     TheDocument.trkmodel.resetAltitudes([i.row() for i in self.ui.trkView.selectionModel().selectedRows()])
     self.ui.statusBar.showMessage(self.tr('Altitudes reset to original values.'), 2000)
 
+  @pyqtSlot(int)
+  def onGotoMainWindow(self, index):
+    model = self.ui.wptView.model()
+    for row in range(model.rowCount()):
+      if model.index(row, 0).data(gpx.IDRole) == index:
+        self.ui.wptView.selectRow(row)
+        self.activateWindow()
+        break
+    else:
+      self.ui.wptView.clearSelection()
+      self.activateWindow()
+      QMessageBox.warning(self, self.tr('Go to point error'), self.tr('This point is hidden because of filter settings.'))
+
   @pyqtSlot()
   def showGoogleMap(self):
     lat = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LAT).data(gpx.ValueRole))
@@ -586,34 +593,17 @@ class GpxMainWindow(QMainWindow):
     webbrowser.open('https://maps.yandex.ru?ll=' + lon + ',' + lat + '&spn=0.03,0.03&pt=' + lon + ',' + lat + '&l=sat')
 
   @pyqtSlot()
-  def showZoomEarthMap(self):
+  def showNakarteMap(self):
     lat = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LAT).data(gpx.ValueRole))
     lon = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LON).data(gpx.ValueRole))
-    webbrowser.open('https://zoom.earth/#' + lat + ',' + lon + ',15z,map')
-
-  @pyqtSlot()
-  def showOpenCycleMap(self):
-    lat = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LAT).data(gpx.ValueRole))
-    lon = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LON).data(gpx.ValueRole))
-    webbrowser.open('https://openstreetmap.org/?mlat=' + lat + '&mlon=' + lon + '&zoom=15&layers=C')
-
-  @pyqtSlot()
-  def showOpenTopoMap(self):
-    lat = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LAT).data(gpx.ValueRole))
-    lon = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LON).data(gpx.ValueRole))
-    webbrowser.open('https://opentopomap.org/#marker=15/' + lat + '/' + lon)
-
-  @pyqtSlot()
-  def showTopoMap(self):
-    lat = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LAT).data(gpx.ValueRole))
-    lon = str(TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.LON).data(gpx.ValueRole))
-    webbrowser.open('http://loadmap.net/ru?q=' + lat + ' ' + lon + '&z=13&s=0')
+    name = TheDocument.wptmodel.index(self.ui.wptView.currentIndex().data(gpx.IDRole), gpx.NAME).data()
+    webbrowser.open('https://nakarte.me/#m=15/' + lat + '/' + lon + '&l=Otm&r=' + lat + '/' + lon + '/' + name)
 
   @pyqtSlot()
   def pointStyle(self):
     dlg = PointConfigDialog(self, self.ui.wptView.currentIndex().data(gpx.IDRole),
                             [i.data(gpx.IDRole) for i in self.ui.wptView.selectionModel().selectedRows()])
-    dlg.exec_()
+    dlg.exec()
 
   @pyqtSlot()
   def skipTracks(self):
@@ -765,14 +755,14 @@ class GpxMainWindow(QMainWindow):
   def openedFileNotFound(self, file):
     result = QMessageBox.warning(self, self.tr('File read error'),
                                  self.tr('The file ') + file + self.tr(' doesn\'t exist.\n\nDo you want to choose another location of this file?'),
-                                 QMessageBox.Yes | QMessageBox.YesToAll | QMessageBox.No, QMessageBox.Yes)
-    if result in (QMessageBox.Yes, QMessageBox.YesToAll):
+                                 QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.StandardButton.YesToAll | QMessageBox.StandardButton.No, QMessageBox.StandardButton.Yes)
+    if result in (QMessageBox.StandardButton.Yes, QMessageBox.StandardButton.YesToAll):
       filename = QFileDialog.getOpenFileName(self, self.tr('Open GPX file') + ' ' + QFileInfo(file).fileName(), TheConfig['MainWindow']['LoadGPXDirectory'],
                                              self.tr('GPX XML (*.gpx *.GPX);;All files (*)'))[0]
       if filename != '':
         TheConfig['MainWindow']['LoadGPXDirectory'] = QFileInfo(filename).path()
         TheDocument.newFilePath = filename
-        if result == QMessageBox.YesToAll:
+        if result == QMessageBox.StandardButton.YesToAll:
           TheDocument.applyToAll = True
 
   @pyqtSlot(list)
@@ -781,8 +771,8 @@ class GpxMainWindow(QMainWindow):
                                                                   + '\n'.join(files)
                                                                   + self.tr('\n\nDo you want to write these files along with the saved project?\n'
                                                                             'NOTE: Some files can be overwritten.'),
-                                 QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
-    if result == QMessageBox.Yes:
+                                 QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No, QMessageBox.StandardButton.Yes)
+    if result == QMessageBox.StandardButton.Yes:
       TheDocument.doExtractFiles = True
 
   @pyqtSlot()
@@ -798,8 +788,8 @@ class GpxMainWindow(QMainWindow):
         else:
           result = QMessageBox.information(self, self.tr('File read error'),
                                            self.tr('The file ') + filename + self.tr(' doesn\'t exist.\n\nDo you want to remove it from recent projects?'),
-                                           QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
-          if result == QMessageBox.Yes:
+                                           QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No, QMessageBox.StandardButton.No)
+          if result == QMessageBox.StandardButton.Yes:
             del TheConfig.recentProjects[i]
             self.updateRecentProjects()
 
@@ -912,7 +902,7 @@ class GpxMainWindow(QMainWindow):
   @pyqtSlot()
   def onShowSettings(self):
     dlg = SettingsDialog(self)
-    if dlg.exec_() == QDialog.Accepted and (TheDocument.wptmodel.rowCount() != 0 or TheDocument.trkmodel.rowCount() != 0):
+    if dlg.exec() == QDialog.DialogCode.Accepted and (TheDocument.wptmodel.rowCount() != 0 or TheDocument.trkmodel.rowCount() != 0):
       self.setProjectChanged()
       TheDocument.gpxparser.updateDistance()
       self.ui.wptView.resizeColumnsToContents()
@@ -925,11 +915,11 @@ class GpxMainWindow(QMainWindow):
     if self.projectChanged and len(TheDocument.doc['GPXFile']) != 0:
       result = QMessageBox.information(self, title,
                                        self.tr('There are unsaved changes. Do you want to save the project?'),
-                                       QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Yes)
-      if result == QMessageBox.Yes:
+                                       QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.Cancel, QMessageBox.StandardButton.Yes)
+      if result == QMessageBox.StandardButton.Yes:
         if not self.onFileSave():
           return False
-      if result == QMessageBox.Cancel:
+      if result == QMessageBox.StandardButton.Cancel:
         return False
 
     return True
@@ -1008,11 +998,16 @@ class GpxMainWindow(QMainWindow):
     self.ui.actionClearList.setDisabled(True)
 
   def initColumnsToCopy(self):
+    if TheConfig.getboolean('MainWindow', 'DetailedView'):
+      TheConfig.columnsToCopy += list(range(gpx.TIME_DAYS, len(gpx.WPTFIELDS)))
+    else:
+      TheConfig.columnsToCopy = [c for c in TheConfig.columnsToCopy if c < gpx.TIME_DAYS]
+
     for act in self.actionsColumns:
       self.ui.menuCoLumns.removeAction(act)
     self.actionsColumns = []
 
-    for i, f in enumerate(TheDocument.wptmodel.fields[:self.filterModel.columnCount()]):
+    for f in TheDocument.wptmodel.fields[:self.filterModel.columnCount()]:
       act = QAction(f, self)
       act.setCheckable(True)
       act.triggered[bool].connect(self.columnsToCopyChanged)
