@@ -1,6 +1,6 @@
 # gpxviewer
 #
-# Copyright (C) 2016-2020 Sergey Salnikov <salsergey@gmail.com>
+# Copyright (C) 2016-2023 Sergey Salnikov <salsergey@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3
@@ -17,6 +17,7 @@
 from datetime import datetime, timedelta
 from lxml import etree
 from math import acos, cos, modf, pi, sin, sqrt
+import re
 from PyQt5.QtCore import Qt, QAbstractTableModel, QFileInfo, QModelIndex, QObject, QPointF, QSortFilterProxyModel, pyqtSignal
 from PyQt5.QtGui import QColor, QFont, QGuiApplication, QPainter, QPainterPath, QPen, QPixmap, QPolygonF
 from gpxviewer.configstore import TheConfig
@@ -25,10 +26,13 @@ from gpxviewer.configstore import TheConfig
 WPTFIELDS = NAME, LAT, LON, ALT, DIST, TIME, TIME_DELTA, TIME_DAYS, DIST_DELTA, ALT_DELTA, SPEED, ALT_SPEED, SLOPE = range(13)
 TRKFIELDS = TRKNAME, TRKSEGS, TRKPTS, TRKLEN, TRKALTGAIN, TRKALTDROP, TRKTIME, TRKDUR = range(8)
 ValueRole, IDRole, IncludeRole, MarkerRole, CaptionRole, SplitLineRole, NeglectRole,\
-  MarkerStyleRole, CaptionStyleRole, SplitLineStyleRole = range(Qt.UserRole, Qt.UserRole + 10)
-MARKER_COLOR, MARKER_STYLE, MARKER_SIZE, CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, LINE_COLOR, LINE_STYLE, LINE_WIDTH = \
-  ('MarkerColor', 'MarkerStyle', 'MarkerSize', 'CaptionPositionX', 'CaptionPositionY',
-   'CaptionRotation', 'CaptionSize', 'SplitLineColor', 'SplitLineStyle', 'SplitLineWidth')
+  MarkerStyleRole, CaptionStyleRole, SplitLineStyleRole = range(Qt.ItemDataRole.UserRole, Qt.ItemDataRole.UserRole + 10)
+MARKER_COLOR, MARKER_STYLE, MARKER_SIZE,\
+  CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC,\
+  LINE_COLOR, LINE_STYLE, LINE_WIDTH = \
+  ('MarkerColor', 'MarkerStyle', 'MarkerSize',
+   'CaptionPositionX', 'CaptionPositionY', 'CaptionRotation', 'CaptionSize', 'CaptionBold', 'CaptionItalic',
+   'SplitLineColor', 'SplitLineStyle', 'SplitLineWidth')
 
 
 class GpxWarning(Exception):
@@ -50,7 +54,7 @@ class WptModel(QAbstractTableModel):
                    self.tr('Speed (km/h)'), self.tr('Climbing speed (m/h)'), self.tr('Slope (m/km)')]
     self.resetModel()
     self.pix = QPixmap(16, 16)
-    self.pix.fill(Qt.transparent)
+    self.pix.fill(Qt.GlobalColor.transparent)
 
     # Define colors that should look well for any color theme
     lightness = max(50, min(240, QGuiApplication.palette().base().color().lightness()))
@@ -70,8 +74,8 @@ class WptModel(QAbstractTableModel):
     else:
       return len(self.fields)
 
-  def data(self, index, role=Qt.DisplayRole):
-    if role == Qt.DisplayRole or role == Qt.EditRole:
+  def data(self, index, role=Qt.ItemDataRole.DisplayRole):
+    if role == Qt.ItemDataRole.DisplayRole or role == Qt.ItemDataRole.EditRole:
       if index.column() == NAME and index.row() in self.changedNames:
         return self.changedNames[index.row()]
       elif index.column() == TIME and self.waypoints[index.row()][index.column()] != '':
@@ -80,15 +84,15 @@ class WptModel(QAbstractTableModel):
         if TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 0:  # Decimal degrees
           return str(round(self.waypoints[index.row()][index.column()], 6))
         elif TheConfig.getValue('ProfileStyle', 'CoordinateFormat') == 1:  # Degrees with decimal minutes
-          min, deg = modf(self.waypoints[index.row()][index.column()])
-          min = round(min * 60, 4)
-          return str(int(deg)) + '° ' + str(min) + '\''
+          mins, degs = modf(self.waypoints[index.row()][index.column()])
+          mins = round(mins * 60, 4)
+          return str(int(degs)) + '° ' + str(mins) + '\''
         else:  # Degrees, minutes, seconds
-          min, deg = modf(self.waypoints[index.row()][index.column()])
-          min = min * 60
-          sec, min = modf(min)
+          mins, degs = modf(self.waypoints[index.row()][index.column()])
+          mins = mins * 60
+          sec, mins = modf(mins)
           sec = round(sec * 60, 2)
-          return str(int(deg)) + '°' + str(int(min)) + '\'' + str(sec) + '"'
+          return str(int(degs)) + '°' + str(int(mins)) + '\'' + str(sec) + '"'
       elif index.column() == ALT and index.row() in self.changedAltitudes:
         return str(self.changedAltitudes[index.row()])
       elif index.column() in {ALT, ALT_DELTA, ALT_SPEED, SLOPE} and self.waypoints[index.row()][index.column()] != '':
@@ -97,7 +101,7 @@ class WptModel(QAbstractTableModel):
         return str(round(self.waypoints[index.row()][index.column()], 3))
       else:
         return str(self.waypoints[index.row()][index.column()])
-    elif role == Qt.DecorationRole and index.column() == NAME:
+    elif role == Qt.ItemDataRole.DecorationRole and index.column() == NAME:
       if index.data(MarkerRole) and index.data(IncludeRole):
         return _markerIcon(index.data(MarkerStyleRole)[MARKER_STYLE], index.data(MarkerStyleRole)[MARKER_COLOR])
       else:
@@ -122,16 +126,16 @@ class WptModel(QAbstractTableModel):
       else:
         return {k: TheConfig.getValue('PointStyle', k) for k in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}}
     elif role == CaptionStyleRole:
-      if all([k in self.pointStyles[index.row()] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}]):
-        return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}}
+      if all([k in self.pointStyles[index.row()] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC}]):
+        return {k: self.pointStyles[index.row()][k] for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC}}
       else:
-        return {k: TheConfig.getValue('PointStyle', k) for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}}
+        return {k: TheConfig.getValue('PointStyle', k) for k in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC}}
     elif role == SplitLineStyleRole:
       if all([k in self.pointStyles[index.row()] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}]):
         return {k: self.pointStyles[index.row()][k] for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
       else:
         return {k: TheConfig.getValue('PointStyle', k) for k in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}}
-    elif role == Qt.BackgroundRole:
+    elif role == Qt.ItemDataRole.BackgroundRole:
       if not self.includeStates[index.row()]:
         return self.SkipColor
       elif self.captionStates[index.row()]:
@@ -139,8 +143,8 @@ class WptModel(QAbstractTableModel):
       elif self.markerStates[index.row()]:
         return self.MarkerColor
       else:
-        return Qt.transparent
-    elif role == Qt.FontRole:
+        return Qt.GlobalColor.transparent
+    elif role == Qt.ItemDataRole.FontRole:
       font = QFont()
       if self.splitStates[index.row()]:
         font.setBold(True)
@@ -150,7 +154,7 @@ class WptModel(QAbstractTableModel):
     return None
 
   def setData(self, index, value, role):
-    if index.isValid() and role == Qt.EditRole and index.column() == NAME and value != self.waypoints[index.row()][NAME]:
+    if index.isValid() and role == Qt.ItemDataRole.EditRole and index.column() == NAME and value != self.waypoints[index.row()][NAME]:
       self.changedNames[index.row()] = value
       self.dataChanged.emit(index, index)
       self.wptDataChanged.emit()
@@ -160,13 +164,13 @@ class WptModel(QAbstractTableModel):
 
   def flags(self, index):
     if index.column() == NAME:
-      return super(WptModel, self).flags(index) | Qt.ItemIsEditable
+      return super(WptModel, self).flags(index) | Qt.ItemFlag.ItemIsEditable
     else:
       return super(WptModel, self).flags(index)
 
   def headerData(self, section, orientation, role):
-    if role == Qt.DisplayRole:
-      return self.fields[section] if orientation == Qt.Horizontal else section + 1
+    if role == Qt.ItemDataRole.DisplayRole:
+      return self.fields[section] if orientation == Qt.Orientation.Horizontal else section + 1
     return None
 
   def parent(self, index=None):
@@ -200,7 +204,7 @@ class WptModel(QAbstractTableModel):
   def getPointStyles(self, key):
     if key in {MARKER_COLOR, MARKER_STYLE, MARKER_SIZE}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.markerStates[i]]
-    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}:
+    if key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.captionStates[i]]
     if key in {LINE_COLOR, LINE_STYLE, LINE_WIDTH}:
       return [p[key] for i, p in enumerate(self.pointStyles) if self.splitStates[i]]
@@ -238,7 +242,7 @@ class WptModel(QAbstractTableModel):
   def setCaptionStates(self, IDs, state):
     for i in IDs:
       self.captionStates[i] = state
-      for key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE}:
+      for key in {CAPTION_POSX, CAPTION_POSY, CAPTION_ROTATION, CAPTION_SIZE, CAPTION_BOLD, CAPTION_ITALIC}:
         if key not in self.pointStyles[i]:
           self.pointStyles[i][key] = TheConfig.getValue('PointStyle', key)
       self.dataChanged.emit(self.index(i, 0), self.index(i, self.columnCount()))
@@ -318,8 +322,8 @@ class TrkModel(QAbstractTableModel):
     else:
       return len(self.fields)
 
-  def data(self, index, role=Qt.DisplayRole):
-    if role == Qt.DisplayRole:
+  def data(self, index, role=Qt.ItemDataRole.DisplayRole):
+    if role == Qt.ItemDataRole.DisplayRole:
       if index.column() == TRKTIME and self.tracks[index.row()][index.column()] != '':
         return str(self.tracks[index.row()][index.column()] + timedelta(minutes=TheConfig.getValue('ProfileStyle', 'TimeZoneOffset')))
       elif index.column() in {TRKALTGAIN, TRKALTDROP}:
@@ -330,7 +334,7 @@ class TrkModel(QAbstractTableModel):
         return str(self.tracks[index.row()][index.column()])
     elif role == IncludeRole:
       return self.includeStates[index.row()]
-    elif role == Qt.BackgroundRole:
+    elif role == Qt.ItemDataRole.BackgroundRole:
       return QGuiApplication.palette().base().color() if self.includeStates[index.row()] else self.SkipColor
     return None
 
@@ -341,8 +345,8 @@ class TrkModel(QAbstractTableModel):
       return self.tracks[track]['SEGMENTS'][segment][index][column]
 
   def headerData(self, section, orientation, role):
-    if role == Qt.DisplayRole:
-      return self.fields[section] if orientation == Qt.Horizontal else section + 1
+    if role == Qt.ItemDataRole.DisplayRole:
+      return self.fields[section] if orientation == Qt.Orientation.Horizontal else section + 1
     return None
 
   def copyToClipboard(self, IDs):
@@ -460,6 +464,7 @@ class GpxParser(QObject):
       tag = 'cmt'
     elif TheConfig.getValue('ProfileStyle', 'ReadNameFromTag') == 2:  # Description
       tag = 'desc'
+    name = None
 
     for p in root.iterfind('{%(ns)s}wpt' % self.ns):
       try:
@@ -528,6 +533,7 @@ class GpxParser(QObject):
                               self.tr('is invalid and will be skipped.', 'Track'))
 
   def parseKMLDocument(self, root):
+    name = None
     for element in root.iterchildren():
       if element.tag == '{%(ns)s}Folder' % self.ns:
         self.parseKMLDocument(element)
@@ -541,7 +547,7 @@ class GpxParser(QObject):
               NAME: name.strip() if name is not None else '',
               TIME: datetime.fromisoformat(time[0:-1]) if time is not None else '',
             }
-            point[LON], point[LAT], point[ALT] = eval(element.findtext('.//{%(ns)s}coordinates' % self.ns))
+            point[LON], point[LAT], point[ALT] = [float(n) for n in element.findtext('.//{%(ns)s}coordinates' % self.ns).split(',')]
 
             self.addPointToModel(point)
 
@@ -556,7 +562,7 @@ class GpxParser(QObject):
             track = {TRKNAME: name.strip() if name is not None else ''}
 
             coords = [el.text.split(' ') for el in element.iterfind('.//{%(gx)s}coord' % self.ns)]
-            times = [datetime.fromisoformat(el.text[0:-1]) for el in element.iterfind('.//{%(ns)s}when' % self.ns)]
+            times = [datetime.fromisoformat(el.text[0:19]) for el in element.iterfind('.//{%(ns)s}when' % self.ns)]
             if len(coords) != len(times):
               times = [None] * len(coords)
 
@@ -596,7 +602,7 @@ class GpxParser(QObject):
             track = {TRKNAME: name.strip() if name is not None else ''}
 
             coords = element.findtext('.//{%(ns)s}coordinates' % self.ns)
-            coords = coords.strip().split(' ') if coords is not None else []
+            coords = re.split(r'\s+', coords.strip()) if coords is not None else []
             times = [None] * len(coords)
 
             prev_lat = None
@@ -605,7 +611,7 @@ class GpxParser(QObject):
             segment = []
             for c, t in zip(coords, times):
               point = {TIME: ''}
-              point[LON], point[LAT], point[ALT] = [float(n) for n in eval(c)]
+              point[LON], point[LAT], point[ALT] = [float(n) for n in c.split(',')]
               self.minalt = min(self.minalt, point[ALT])
               self.maxalt = max(self.maxalt, point[ALT])
 
@@ -776,6 +782,7 @@ class GpxParser(QObject):
       return
 
     i = 0
+    prev = {}
     for i, p in enumerate(self.wptmodel.waypoints):
       if self.wptmodel.includeStates[i]:
         prev = p
@@ -819,57 +826,96 @@ class GpxParser(QObject):
     TheConfig['ProfileStyle']['MaximumAltitude'] = str(int(round(self.maxalt, -3) + 500 if round(self.maxalt, -3) < self.maxalt
                                                            else round(self.maxalt, -3)))
 
+  def writePointToGPX(self, root, point, name=None, ele=None):
+    element = etree.SubElement(root, 'wpt', lat=str(point[LAT]), lon=str(point[LON]))
+    etree.SubElement(element, 'ele').text = self.wptmodel.index(point['ID'], ALT).data() if ele is None else ele
+    self.minlat = min(self.minlat, point[LAT])
+    self.minlon = min(self.minlon, point[LON])
+    self.maxlat = max(self.maxlat, point[LAT])
+    self.maxlon = max(self.maxlon, point[LON])
+    if point[TIME] != '':
+      etree.SubElement(element, 'time').text = point[TIME].isoformat() + 'Z'
+    etree.SubElement(element, 'name').text = self.wptmodel.index(point['ID'], NAME).data() if name is None else name
+
+    if 'CMT' in point and point['CMT'] is not None:
+      etree.SubElement(element, 'cmt').text = point['CMT']
+    if 'DESC' in point and point['DESC'] is not None:
+      etree.SubElement(element, 'desc').text = point['DESC']
+    if 'SYM' in point and point['SYM'] is not None:
+      etree.SubElement(element, 'sym').text = point['SYM']
+
+  def writePointToGPXTrack(self, root, point, ele):
+    element = etree.SubElement(root, 'trkpt', lat=str(point[LAT]), lon=str(point[LON]))
+    etree.SubElement(element, 'ele').text = ele
+    self.minlat = min(self.minlat, point[LAT])
+    self.minlon = min(self.minlon, point[LON])
+    self.maxlat = max(self.maxlat, point[LAT])
+    self.maxlon = max(self.maxlon, point[LON])
+    if point[TIME] != '':
+      etree.SubElement(element, 'time').text = point[TIME].isoformat() + 'Z'
+
   def writeGPXFile(self, filename):
     ns = {None: 'http://www.topografix.com/GPX/1/1'}
     root = etree.Element('gpx', nsmap=ns, version='1.1', creator='GPX Viewer - https://osdn.net/projects/gpxviewer')
     metadata = etree.SubElement(root, 'metadata')
 
-    minlat = 90
-    minlon = 180
-    maxlat = 0
-    maxlon = -180
+    self.minlat = 90
+    self.minlon = 180
+    self.maxlat = -90
+    self.maxlon = -180
 
-    for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
-      if state:
-        element = etree.SubElement(root, 'wpt', lat=str(point[LAT]), lon=str(point[LON]))
-        etree.SubElement(element, 'ele').text = self.wptmodel.index(point['ID'], ALT).data()
-        minlat = min(minlat, point[LAT])
-        minlon = min(minlon, point[LON])
-        maxlat = max(maxlat, point[LAT])
-        maxlon = max(maxlon, point[LON])
-        if point[TIME] != '':
-          etree.SubElement(element, 'time').text = point[TIME].isoformat() + 'Z'
-        etree.SubElement(element, 'name').text = self.wptmodel.index(point['ID'], NAME).data()
+    if not TheConfig.getValue('ProfileStyle', 'DeletePoints'):
+      for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
+        if state:
+          self.writePointToGPX(root, point)
 
-        if 'CMT' in point and point['CMT'] is not None:
-          etree.SubElement(element, 'cmt').text = point['CMT']
-        if 'DESC' in point and point['DESC'] is not None:
-          etree.SubElement(element, 'desc').text = point['DESC']
-        if 'SYM' in point and point['SYM'] is not None:
-          etree.SubElement(element, 'sym').text = point['SYM']
+    if TheConfig.getValue('ProfileStyle', 'TracksToPoints'):
+      num = self.wptmodel.rowCount() - len(self.wptmodel.getSkippedPoints())
+      for tnum, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.includeStates):
+        if state:
+          for snum, seg in enumerate(self.trkmodel.tracks[tnum]['SEGMENTS']):
+            for i, point in enumerate(seg):
+              num += 1
+              self.writePointToGPX(root, point,
+                                   name='WPT{}'.format(num),
+                                   ele=str(self.trkmodel.getPointData(tnum, snum, i, ALT)))
 
-    for tnum, track, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.tracks, self.trkmodel.includeStates):
-      if state:
-        elTrk = etree.SubElement(root, 'trk')
-        etree.SubElement(elTrk, 'name').text = track[TRKNAME]
-        for snum, seg in enumerate(track['SEGMENTS']):
-          elSeg = etree.SubElement(elTrk, 'trkseg')
-          for i, point in enumerate(seg):
-            elP = etree.SubElement(elSeg, 'trkpt', lat=str(point[LAT]), lon=str(point[LON]))
-            etree.SubElement(elP, 'ele').text = str(self.trkmodel.getPointData(tnum, snum, i, ALT))
-            minlat = min(minlat, point[LAT])
-            minlon = min(minlon, point[LON])
-            maxlat = max(maxlat, point[LAT])
-            maxlon = max(maxlon, point[LON])
-            if point[TIME] != '':
-              etree.SubElement(elP, 'time').text = point[TIME].isoformat() + 'Z'
+    if not TheConfig.getValue('ProfileStyle', 'DeleteTracks'):
+      for tnum, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.includeStates):
+        if state:
+          elTrk = etree.SubElement(root, 'trk')
+          track = self.trkmodel.tracks[tnum]
+          etree.SubElement(elTrk, 'name').text = track[TRKNAME]
+          for snum, seg in enumerate(track['SEGMENTS']):
+            elSeg = etree.SubElement(elTrk, 'trkseg')
+            for i, point in enumerate(seg):
+              self.writePointToGPXTrack(elSeg, point, ele=str(self.trkmodel.getPointData(tnum, snum, i, ALT)))
+
+    if TheConfig.getValue('ProfileStyle', 'PointsToTrack'):
+      elTrk = etree.SubElement(root, 'trk')
+      etree.SubElement(elTrk, 'name').text = self.tr('Waypoints')
+      elSeg = etree.SubElement(elTrk, 'trkseg')
+      for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
+        if state:
+          self.writePointToGPXTrack(elSeg, point, ele=self.wptmodel.index(point['ID'], ALT).data())
 
     etree.SubElement(metadata, 'time').text = datetime.utcnow().isoformat() + 'Z'
-    etree.SubElement(metadata, 'bounds', minlat=str(minlat), minlon=str(minlon), maxlat=str(maxlat), maxlon=str(maxlon))
+    etree.SubElement(metadata, 'bounds', minlat=str(self.minlat), minlon=str(self.minlon), maxlat=str(self.maxlat), maxlon=str(self.maxlon))
 
     with open(filename, 'w', encoding='utf-8') as file:
       file.write('<?xml version="1.0" encoding="UTF-8"?>\n' + etree.tostring(root, encoding='unicode', pretty_print=True))
 
+  def writePointToKML(self, root, point, name=None, ele=None):
+    place = etree.SubElement(root, 'Placemark')
+    etree.SubElement(place, 'name').text = self.wptmodel.index(point['ID'], NAME).data() if name is None else name
+    if point[TIME] != '':
+      ts = etree.SubElement(place, 'TimeStamp')
+      etree.SubElement(ts, 'when').text = point[TIME].isoformat() + 'Z'
+    p = etree.SubElement(place, 'Point')
+    etree.SubElement(p, 'coordinates').text = '{},{},{}'.format(str(point[LON]),
+                                                                str(point[LAT]),
+                                                                self.wptmodel.index(point['ID'], ALT).data() if ele is None else ele)
+
   def writeKMLFile(self, filename):
     ns = {
       None: 'http://www.opengis.net/kml/2.2',
@@ -904,42 +950,75 @@ class GpxParser(QObject):
     trkfolder = etree.SubElement(document, 'Folder')
     etree.SubElement(trkfolder, 'name').text = 'Tracks'
 
-    for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
-      if state:
-        place = etree.SubElement(wptfolder, 'Placemark')
-        etree.SubElement(place, 'name').text = self.wptmodel.index(point['ID'], NAME).data()
-        if point[TIME] != '':
-          ts = etree.SubElement(place, 'TimeStamp')
-          etree.SubElement(ts, 'when').text = point[TIME].isoformat() + 'Z'
-        p = etree.SubElement(place, 'Point')
-        etree.SubElement(p, 'coordinates').text = str(point[LON]) + ',' + str(point[LAT]) + ',' + self.wptmodel.index(point['ID'], ALT).data()
+    if not TheConfig.getValue('ProfileStyle', 'DeletePoints'):
+      for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
+        if state:
+          self.writePointToKML(wptfolder, point)
 
-    for tnum, track, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.tracks, self.trkmodel.includeStates):
-      if state:
-        place = etree.SubElement(trkfolder, 'Placemark')
-        etree.SubElement(place, 'name').text = track[TRKNAME]
-        etree.SubElement(place, 'styleUrl').text = '#track'
-
-        if track[TRKTIME] != '':
-          tr = etree.SubElement(place, '{%(gx)s}Track' % ns)
-          times = []
-          coords = []
-          for snum, seg in enumerate(track['SEGMENTS']):
+    if TheConfig.getValue('ProfileStyle', 'TracksToPoints'):
+      num = self.wptmodel.rowCount() - len(self.wptmodel.getSkippedPoints())
+      for tnum, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.includeStates):
+        if state:
+          for snum, seg in enumerate(self.trkmodel.tracks[tnum]['SEGMENTS']):
             for i, point in enumerate(seg):
-              times += [point[TIME].isoformat() + 'Z']
-              coords += [str(point[LON]) + ' ' + str(point[LAT]) + ' ' + str(self.trkmodel.getPointData(tnum, snum, i, ALT))]
-          for t in times:
-            etree.SubElement(tr, 'when').text = t
-          for c in coords:
-            etree.SubElement(tr, '{%(gx)s}coord' % ns).text = c
+              num += 1
+              self.writePointToKML(wptfolder, point,
+                                   name='WPT{}'.format(num),
+                                   ele=str(self.trkmodel.getPointData(tnum, snum, i, ALT)))
 
-        else:
-          tr = etree.SubElement(place, 'LineString')
-          coords = []
-          for snum, seg in enumerate(track['SEGMENTS']):
-            for i, point in enumerate(seg):
-              coords += [str(point[LON]) + ',' + str(point[LAT]) + ',' + str(self.trkmodel.getPointData(tnum, snum, i, ALT))]
-          etree.SubElement(tr, 'coordinates').text = ' '.join(coords)
+    if not TheConfig.getValue('ProfileStyle', 'DeleteTracks'):
+      for tnum, track, state in zip(range(len(self.trkmodel.tracks)), self.trkmodel.tracks, self.trkmodel.includeStates):
+        if state:
+          place = etree.SubElement(trkfolder, 'Placemark')
+          etree.SubElement(place, 'name').text = track[TRKNAME]
+          etree.SubElement(place, 'styleUrl').text = '#track'
+
+          if track[TRKTIME] != '':
+            tr = etree.SubElement(place, '{%(gx)s}Track' % ns)
+            times = []
+            coords = []
+            for snum, seg in enumerate(track['SEGMENTS']):
+              for i, point in enumerate(seg):
+                times += [point[TIME].isoformat() + 'Z']
+                coords += ['{} {} {}'.format(str(point[LON]), str(point[LAT]), str(self.trkmodel.getPointData(tnum, snum, i, ALT)))]
+            for t in times:
+              etree.SubElement(tr, 'when').text = t
+            for c in coords:
+              etree.SubElement(tr, '{%(gx)s}coord' % ns).text = c
+
+          else:  # track[TRKTIME] == ''
+            tr = etree.SubElement(place, 'LineString')
+            coords = []
+            for snum, seg in enumerate(track['SEGMENTS']):
+              for i, point in enumerate(seg):
+                coords += ['{},{},{}'.format(str(point[LON]), str(point[LAT]), str(self.trkmodel.getPointData(tnum, snum, i, ALT)))]
+            etree.SubElement(tr, 'coordinates').text = ' '.join(coords)
+
+    if TheConfig.getValue('ProfileStyle', 'PointsToTrack'):
+      place = etree.SubElement(trkfolder, 'Placemark')
+      etree.SubElement(place, 'name').text = self.tr('Waypoints')
+      etree.SubElement(place, 'styleUrl').text = '#track'
+
+      if self.wptmodel.waypoints[0][TIME] != '':
+        tr = etree.SubElement(place, '{%(gx)s}Track' % ns)
+        times = []
+        coords = []
+        for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
+          if state:
+            times += [point[TIME].isoformat() + 'Z']
+            coords += ['{} {} {}'.format(str(point[LON]), str(point[LAT]), self.wptmodel.index(point['ID'], ALT).data())]
+        for t in times:
+          etree.SubElement(tr, 'when').text = t
+        for c in coords:
+          etree.SubElement(tr, '{%(gx)s}coord' % ns).text = c
+
+      else:  # self.wptmodel.waypoints[0][TIME] == ''
+        tr = etree.SubElement(place, 'LineString')
+        coords = []
+        for point, state in zip(self.wptmodel.waypoints, self.wptmodel.includeStates):
+          if state:
+            coords += ['{},{},{}'.format(str(point[LON]), str(point[LAT]), self.wptmodel.index(point['ID'], ALT).data())]
+        etree.SubElement(tr, 'coordinates').text = ' '.join(coords)
 
     with open(filename, 'w', encoding='utf-8') as file:
       file.write('<?xml version="1.0" encoding="UTF-8"?>\n' + etree.tostring(root, encoding='unicode', pretty_print=True))
@@ -948,7 +1027,7 @@ class GpxParser(QObject):
 
 
 class GpxSortFilterModel(QSortFilterProxyModel):
-  def __init__(self, parent):
+  def __init__(self, parent=None):
     super(GpxSortFilterModel, self).__init__(parent)
 
     self.includeSkipped = True
@@ -1156,13 +1235,13 @@ def markerPath(style, size):
 def _markerIcon(style, color):
   size = 16
   pix = QPixmap(size, size)
-  pix.fill(Qt.transparent)
+  pix.fill(Qt.GlobalColor.transparent)
   p = QPainter(pix)
-  p.setRenderHint(QPainter.Antialiasing)
+  p.setRenderHint(QPainter.RenderHint.Antialiasing)
   if style in {'1', '2', '3', '4', 'ad', 'au', 'al', 'ar', '+', 'x', '_', '|'}:
-    p.setPen(QPen(QColor(color), 3, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+    p.setPen(QPen(QColor(color), 3, Qt.PenStyle.SolidLine, Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin))
   else:
-    p.setPen(QPen(QColor(color), 1, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))
+    p.setPen(QPen(QColor(color), 1, Qt.PenStyle.SolidLine, Qt.PenCapStyle.RoundCap, Qt.PenJoinStyle.RoundJoin))
   p.setBrush(QColor(color))
   p.drawPath(markerPath(style, 0.8 * size).translated(QPointF(size / 2.0, size / 2.0)))
 
